@@ -10,7 +10,7 @@ local type         = type
 ---@field type                  string
 ---@field special               string
 ---@field tag                   string
----@field args                  { [integer]: parser.object, start: integer, finish: integer }
+---@field args                  { [integer]: parser.object, start: integer, finish: integer, type: string }
 ---@field locals                parser.object[]
 ---@field returns?              parser.object[]
 ---@field breaks?               parser.object[]
@@ -21,6 +21,7 @@ local type         = type
 ---@field finish                integer
 ---@field range                 integer
 ---@field effect                integer
+---@field bstart                integer
 ---@field attrs                 string[]
 ---@field specials              parser.object[]
 ---@field labels                parser.object[]
@@ -55,7 +56,7 @@ local type         = type
 ---@field returnIndex           integer
 ---@field assignIndex           integer
 ---@field docIndex              integer
----@field docs                  parser.object[]
+---@field docs                  parser.object
 ---@field state                 table
 ---@field comment               table
 ---@field optional              boolean
@@ -71,9 +72,14 @@ local type         = type
 ---@field hasGoTo?              true
 ---@field hasReturn?            true
 ---@field hasBreak?             true
----@field hasError?             true
+---@field hasExit?              true
 ---@field [integer]             parser.object|any
----@field package _root          parser.object
+---@field dot                   { type: string, start: integer, finish: integer }
+---@field colon                 { type: string, start: integer, finish: integer }
+---@field package _root         parser.object
+---@field package _eachCache?   parser.object[]
+---@field package _isGlobal?    boolean
+---@field package _typeCache?   parser.object[][]
 
 ---@class guide
 ---@field debugMode boolean
@@ -81,7 +87,8 @@ local m = {}
 
 m.ANY = {"<ANY>"}
 
-m.namePattern = '[%a_\x80-\xff][%w_\x80-\xff]*'
+m.notNamePattern  = '[^%w_\x80-\xff]'
+m.namePattern     = '[%a_\x80-\xff][%w_\x80-\xff]*'
 m.namePatternFull = '^' .. m.namePattern .. '$'
 
 local blockTypes = {
@@ -98,6 +105,18 @@ local blockTypes = {
     ['main']        = true,
 }
 
+m.blockTypes = blockTypes
+
+local topBlockTypes = {
+    ['while']       = true,
+    ['function']    = true,
+    ['if']          = true,
+    ['ifblock']     = true,
+    ['elseblock']   = true,
+    ['elseifblock'] = true,
+    ['main']        = true,
+}
+
 local breakBlockTypes = {
     ['while']       = true,
     ['in']          = true,
@@ -112,40 +131,40 @@ local childMap = {
     ['while']       = {'filter', '#'},
     ['in']          = {'keys', 'exps', '#'},
     ['loop']        = {'loc', 'init', 'max', 'step', '#'},
+    ['do']          = {'#'},
     ['if']          = {'#'},
     ['ifblock']     = {'filter', '#'},
     ['elseifblock'] = {'filter', '#'},
     ['elseblock']   = {'#'},
     ['setfield']    = {'node', 'field', 'value'},
-    ['setglobal']   = {'value'},
-    ['local']       = {'attrs', 'value'},
-    ['setlocal']    = {'value'},
-    ['return']      = {'#'},
-    ['do']          = {'#'},
-    ['select']      = {'vararg'},
-    ['table']       = {'#'},
-    ['tableindex']  = {'index', 'value'},
-    ['tablefield']  = {'field', 'value'},
-    ['tableexp']    = {'value'},
-    ['function']    = {'args', '#'},
-    ['funcargs']    = {'#'},
+    ['getfield']    = {'node', 'field'},
     ['setmethod']   = {'node', 'method', 'value'},
     ['getmethod']   = {'node', 'method'},
     ['setindex']    = {'node', 'index', 'value'},
     ['getindex']    = {'node', 'index'},
+    ['tableindex']  = {'index', 'value'},
+    ['tablefield']  = {'field', 'value'},
+    ['tableexp']    = {'value'},
+    ['setglobal']   = {'value'},
+    ['local']       = {'attrs', 'value'},
+    ['setlocal']    = {'value'},
+    ['return']      = {'#'},
+    ['select']      = {'vararg'},
+    ['table']       = {'#'},
+    ['function']    = {'args', '#'},
+    ['funcargs']    = {'#'},
     ['paren']       = {'exp'},
     ['call']        = {'node', 'args'},
     ['callargs']    = {'#'},
-    ['getfield']    = {'node', 'field'},
     ['list']        = {'#'},
     ['binary']      = {1, 2},
     ['unary']       = {1},
 
     ['doc']                = {'#'},
-    ['doc.class']          = {'class', '#extends', '#signs', 'comment'},
+    ['doc.class']          = {'class', '#extends', '#signs', 'docAttr', 'comment'},
     ['doc.type']           = {'#types', 'name', 'comment'},
-    ['doc.alias']          = {'alias', 'extends', 'comment'},
-    ['doc.enum']           = {'enum', 'extends', 'comment'},
+    ['doc.alias']          = {'alias', 'docAttr', 'extends', 'comment'},
+    ['doc.enum']           = {'enum', 'extends', 'comment', 'docAttr'},
     ['doc.param']          = {'param', 'extends', 'comment'},
     ['doc.return']         = {'#returns', 'comment'},
     ['doc.field']          = {'field', 'extends', 'comment'},
@@ -164,9 +183,11 @@ local childMap = {
     ['doc.version']        = {'#versions'},
     ['doc.diagnostic']     = {'#names'},
     ['doc.as']             = {'as'},
-    ['doc.cast']           = {'loc', '#casts'},
+    ['doc.cast']           = {'name', '#casts'},
     ['doc.cast.block']     = {'extends'},
     ['doc.operator']       = {'op', 'exp', 'extends'},
+    ['doc.meta']           = {'name'},
+    ['doc.attr']           = {'#names'},
 }
 
 ---@type table<string, fun(obj: parser.object, list: parser.object[])>
@@ -406,6 +427,22 @@ function m.getParentType(obj, want)
     error('guide.getParentType overstack')
 end
 
+--- 寻找所在父类型
+---@param obj parser.object
+---@return parser.object?
+function m.getParentTypes(obj, wants)
+    for _ = 1, 10000 do
+        obj = obj.parent
+        if not obj then
+            return nil
+        end
+        if wants[obj.type] then
+            return obj
+        end
+    end
+    error('guide.getParentTypes overstack')
+end
+
 --- 寻找根区块
 ---@param obj parser.object
 ---@return parser.object
@@ -432,7 +469,7 @@ function m.getRoot(obj)
     error('guide.getRoot overstack')
 end
 
----@param obj parser.object
+---@param obj parser.object | { uri: uri }
 ---@return uri
 function m.getUri(obj)
     if obj.uri then
@@ -466,6 +503,9 @@ function m.getLocal(source, name, pos)
         if not block then
             return nil
         end
+        if block.type == 'main' then
+            break
+        end
         if  block.start <= pos
         and block.finish >= pos
         and blockTypes[block.type] then
@@ -704,7 +744,7 @@ end
 --- 遍历所有指定类型的source
 ---@param ast parser.object
 ---@param type string
----@param callback fun(src: parser.object)
+---@param callback fun(src: parser.object): any
 ---@return any
 function m.eachSourceType(ast, type, callback)
     local cache = getSourceTypeCache(ast)
@@ -883,7 +923,7 @@ function m.getLineRange(state, row)
     return 0
 end
 
-local isSetMap = {
+local assignTypeMap = {
     ['setglobal']         = true,
     ['local']             = true,
     ['self']              = true,
@@ -893,7 +933,6 @@ local isSetMap = {
     ['setindex']          = true,
     ['tablefield']        = true,
     ['tableindex']        = true,
-    ['tableexp']          = true,
     ['label']             = true,
     ['doc.class']         = true,
     ['doc.alias']         = true,
@@ -906,9 +945,9 @@ local isSetMap = {
     ['doc.type.field']    = true,
     ['doc.type.array']    = true,
 }
-function m.isSet(source)
+function m.isAssign(source)
     local tp = source.type
-    if isSetMap[tp] then
+    if assignTypeMap[tp] then
         return true
     end
     if tp == 'call' then
@@ -920,7 +959,7 @@ function m.isSet(source)
     return false
 end
 
-local isGetMap = {
+local getTypeMap = {
     ['getglobal'] = true,
     ['getlocal']  = true,
     ['getfield']  = true,
@@ -929,7 +968,7 @@ local isGetMap = {
 }
 function m.isGet(source)
     local tp = source.type
-    if isGetMap[tp] then
+    if getTypeMap[tp] then
         return true
     end
     if tp == 'call' then
@@ -1009,7 +1048,12 @@ function m.getKeyName(obj)
         return obj.enum[1]
     elseif tp == 'doc.field' then
         return obj.field[1]
-    elseif tp == 'doc.field.name' then
+    elseif tp == 'doc.field.name'
+    or     tp == 'doc.type.name'
+    or     tp == 'doc.class.name'
+    or     tp == 'doc.alias.name'
+    or     tp == 'doc.enum.name'
+    or     tp == 'doc.extends.name' then
         return obj[1]
     elseif tp == 'doc.type.field' then
         return m.getKeyName(obj.name)
@@ -1082,99 +1126,6 @@ function m.getKeyType(obj)
     return m.getKeyTypeOfLiteral(obj)
 end
 
---- 测试 a 到 b 的路径（不经过函数，不考虑 goto），
---- 每个路径是一个 block 。
----
---- 如果 a 在 b 的前面，返回 `"before"` 加上 2个`list<block>`
----
---- 如果 a 在 b 的后面，返回 `"after"` 加上 2个`list<block>`
----
---- 否则返回 `false`
----
---- 返回的2个 `list` 分别为基准block到达 a 与 b 的路径。
----@param a table
----@param b table
----@return string|false mode
----@return table? pathA
----@return table? pathB
-function m.getPath(a, b, sameFunction)
-    --- 首先测试双方在同一个函数内
-    if sameFunction and m.getParentFunction(a) ~= m.getParentFunction(b) then
-        return false
-    end
-    local mode
-    local objA
-    local objB
-    if a.finish < b.start then
-        mode = 'before'
-        objA = a
-        objB = b
-    elseif a.start > b.finish then
-        mode = 'after'
-        objA = b
-        objB = a
-    else
-        return 'equal', {}, {}
-    end
-    local pathA = {}
-    local pathB = {}
-    for _ = 1, 1000 do
-        objA = m.getParentBlock(objA)
-        if objA then
-            pathA[#pathA+1] = objA
-            if (not sameFunction and objA.type == 'function') or objA.type == 'main' then
-                break
-            end
-        end
-    end
-    for _ = 1, 1000 do
-        objB = m.getParentBlock(objB)
-        if objB then
-            pathB[#pathB+1] = objB
-            if (not sameFunction and objB.type == 'function') or objB.type == 'main' then
-                break
-            end
-        end
-    end
-    -- pathA: {1, 2, 3, 4, 5}
-    -- pathB: {5, 6, 2, 3}
-    local top = #pathB
-    local start
-    for i = #pathA, 1, -1 do
-        local currentBlock = pathA[i]
-        if currentBlock == pathB[top] then
-            start = i
-            break
-        end
-    end
-    if not start then
-        return false
-    end
-    -- pathA: {   1, 2, 3}
-    -- pathB: {5, 6, 2, 3}
-    local extra = 0
-    local align = top - start
-    for i = start, 1, -1 do
-        local currentA = pathA[i]
-        local currentB = pathB[i+align]
-        if currentA ~= currentB then
-            extra = i
-            break
-        end
-    end
-    -- pathA: {1}
-    local resultA = {}
-    for i = extra, 1, -1 do
-        resultA[#resultA+1] = pathA[i]
-    end
-    -- pathB: {5, 6}
-    local resultB = {}
-    for i = extra + align, 1, -1 do
-        resultB[#resultB+1] = pathB[i]
-    end
-    return mode, resultA, resultB
-end
-
 ---是否是全局变量（包括 _G.XXX 形式）
 ---@param source parser.object
 ---@return boolean
@@ -1333,4 +1284,64 @@ function m.getFunctionSelfNode(func)
     return nil
 end
 
+---@param source parser.object
+---@return parser.object?
+function m.getTopBlock(source)
+    for _ = 1, 1000 do
+        local block = source.parent
+        if not block then
+            return nil
+        end
+        if topBlockTypes[block.type] then
+            return block
+        end
+        source = block
+    end
+    return nil
+end
+
+---@param source parser.object
+---@return boolean
+function m.isParam(source)
+    if  source.type ~= 'local'
+    and source.type ~= 'self' then
+        return false
+    end
+    if source.parent.type ~= 'funcargs' then
+        return false
+    end
+    return true
+end
+
+---@param source parser.object
+---@return parser.object[]?
+function m.getParams(source)
+    if source.type == 'call' then
+        local args = source.args
+        if not args then
+            return
+        end
+        assert(args.type == 'callargs', 'call.args type is\'t callargs')
+        return args
+    elseif source.type == 'callargs' then
+        return source
+    elseif source.type == 'function' then
+        local args = source.args
+        if not args then
+            return
+        end
+        assert(args.type == 'funcargs', 'function.args type is\'t callargs')
+        return args
+    end
+    return nil
+end
+
+---@param source parser.object
+---@param index integer
+---@return parser.object?
+function m.getParam(source, index)
+    local args = m.getParams(source)
+    return args and args[index] or nil
+end
+
 return m
